<?php

/*
+---------------------------------------------------------------------------+
| Max Media Manager v0.3                                                    |
| =================                                                         |
|                                                                           |
| Copyright (c) 2003-2005 m3 Media Services Limited                         |
| For contact details, see: http://www.m3.net/                              |
|                                                                           |
| This program is free software; you can redistribute it and/or modify      |
| it under the terms of the GNU General Public License as published by      |
| the Free Software Foundation; either version 2 of the License, or         |
| (at your option) any later version.                                       |
|                                                                           |
| This program is distributed in the hope that it will be useful,           |
| but WITHOUT ANY WARRANTY; without even the implied warranty of            |
| MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the             |
| GNU General Public License for more details.                              |
|                                                                           |
| You should have received a copy of the GNU General Public License         |
| along with this program; if not, write to the Free Software               |
| Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA |
+---------------------------------------------------------------------------+
$Id: Maintenance.php 3461 2005-07-15 16:11:10Z andrew $
*/

require_once MAX_PATH . '/lib/core/ServiceLocator.php';
require_once MAX_PATH . '/lib/max/DB.php';
require_once MAX_PATH . '/lib/max/Maintenance/OperationInterval.php';
require_once 'Date.php';
require_once 'DB/QueryTool.php';

/**
 * The base, non-DB specific Data Access Layer (DAL) class for the Maintenace
 * and Forecasting engines.
 *
 * @package    MaxDal
 * @author     Radek Maciaszek <radek@m3.net>
 */
class MAX_Dal_Common
{
    var $dbh;
    var $queryBuilder;
    var $prefix;
    
    /**
     * The class constructor method.
     */
    function MAX_Dal_Common()
    {
        $this->conf = $GLOBALS['_MAX']['CONF'];
        $this->prefix = $GLOBALS['_MAX']['CONF']['table']['prefix'];
        $this->dbh = &$this->_getDbConnection();
        $dsn = MAX_DB::getDsn(MAX_DSN_STRING);
        $this->queryBuilder = $this->_getQueryTool($dsn);
    }
    
    /**
     * A private method to manage creation of the utilised MAX_DB class.
     *
     * @access private
     * @return mixed An instance of the MAX_DB class.
     */
    function &_getDbConnection()
    {
        $oServiceLocator = &ServiceLocator::instance();
        $dbh = $oServiceLocator->get('MAX_DB');
        if (!$dbh) {
            $dbh = &MAX_DB::singleton();
        }
        return $dbh;
    }
    
    /**
     * A private method to manage creation of the Maintenance class.
     *
     * @access private
     * @return mixed An instance of the Maintenance class.
     */
    function &_getMaintenance()
    {
        $oServiceLocator = &ServiceLocator::instance();
        $maintenance = $oServiceLocator->get('Maintenance');
        if (!$maintenance) {
            $maintenance = &new Maintenance();
        }
        return $maintenance;
    }
    
    /**
     * A private method for instantiating the DB_QueryTool class.
     *
     * @access private
     * @param string $dsn The DSN string for the database connection.
     */
    function _getQueryTool($dsn)
    {
        return new DB_QueryTool($dsn);
    }

    /**
     * A private method used for generating data access SQL via the DB_QueryTool,
     * and returning the results.
     *
     * @access private
     * @param array $aParams A hash of parameters, indexed on the following strings:
     *  - "table"    The (primary) database table to select from.
     *  - "fields"   An optional array of the "table.column"s (in all tables) to
     *               select, default is all columns.
     *  - "wheres"   An optional array of arrays, each containing a two strings -
     *               the first being the where clause, and the second being 'AND'
     *               or 'OR'. Bracketing and AND/OR can be used inside the where
     *               string to make complex logic if needed.
     *  - "joins"    An optional array of arrays, each containing two strings - the
     *               first being the table to join to (ie. not $table) and the second
     *               being the complete where statement to link the tables (ie.
     *               do not have more than one join array for the same extra table).
     *  - "group"    An optional comma separated list of "table.column"s to group by.
     *  - "orderBys" An optional array of arrays, each containing two strings - the
     *               first being a "table.column" to order by, and the second being
     *               either "ASC" or "DESC".
     *  - "havings"  An optional array of arrays, each containing a two strings -
     *               the first being the having clause, and the second being 'AND'
     *               or 'OR'. Bracketing and AND/OR can be used inside the having
     *               string to make complex logic if needed.
     * @return array An array of arrays representing the result(s) of the query.
     */
    function &_get($aParams)
    {
        if (is_null($aParams['table'])) {
            return array();
        }
        // Reset the query builder
        $this->queryBuilder->reset();
        // Set the primary table
        $this->queryBuilder->setTable($aParams['table']);
        // Add fields
        if (count($aParams['fields']) > 0) {
            $this->queryBuilder->setSelect($aParams['fields'][0]);
            if (count($aParams['fields']) > 1) {
                for ($counter = 1; $counter < count($aParams['fields']); $counter++) {
                    $this->queryBuilder->addSelect($aParams['fields'][$counter]);
                }
            }
        }
        // Add where conditions
        if (count($aParams['wheres']) > 0) {
            list($constraint, $operator) = $aParams['wheres'][0];
            $this->queryBuilder->setWhere($constraint, $operator);
            if (count($aParams['wheres']) > 1) {
                for ($counter = 1; $counter < count($aParams['wheres']); $counter++) {
                    list($constraint, $operator) = $aParams['wheres'][$counter];
                    $this->queryBuilder->addWhere($constraint, $operator);
                }
            }
        }
        // Add joins
        if (!is_null($aParams['joins'])) {
            foreach ($aParams['joins'] as $join) {
                list($table, $joinCond) = $join;
                $this->queryBuilder->addJoin($table, $joinCond);
            }
        }
        // Add the grouping
        if (!is_null($aParams['group'])) {
            $this->queryBuilder->setGroup($aParams['group']);
        }
        // Add order by conditions
        if (count($aParams['orderBys']) > 0) {
            list($field, $direction) = $aParams['orderBys'][0];
            if (!is_bool($direction)) {
                $direction = ($direction == 'DESC') ? true : false;
            }
            $this->queryBuilder->setOrder($field, $direction);
            if (count($aParams['orderBys']) > 1) {
                for ($counter = 1; $counter < count($aParams['orderBys']); $counter++) {
                    list($field, $direction) = $aParams['orderBys'][$counter];
                    if (!is_bool($direction)) {
                        $direction = ($direction == 'DESC') ? true : false;
                    }
                    $this->queryBuilder->addOrder($field, $direction);
                }
            }
        }
        // Add having conditions
        if (count($aParams['havings']) > 0) {
            list($constraint, $operator) = $aParams['havings'][0];
            $this->queryBuilder->setHaving($constraint);
            if (count($aParams['havings']) > 1) {
                for ($counter = 1; $counter < count($aParams['havings']); $counter++) {
                    list($constraint, $operator) = $aParams['havings'][$counter];
                    $this->queryBuilder->addHaving($constraint, $operator);
                }
            }
        }
        // Run the query and return the result(s)
        return $this->queryBuilder->getAll();
    }
    
    /**
     * A method to store the last time that a Maintenance or Forecasting script
     * ran.
     *
     * @param Date $start The time that the script run started.
     * @param Date $end The time that the script run ended.
     * @param Date $updateTo The end of the last operation interval ID that
     *                       has been updated.
     * @param string $tableName The name of Maintenance or Forecasting log table
     */
    function setScriptLastRunInfo($start, $end, $updateTo, $tableName)
    {
        $conf = $GLOBALS['_MAX']['CONF'];
        $duration = new Date_Span();
        $oStartDateCopy = &new Date($start); // Clone for Date_Span::setFromDateDiff bug in PHP5
        $oEndDateCopy   = &new Date($end);   // Clone for Date_Span::setFromDateDiff bug in PHP5
        $duration->setFromDateDiff($oStartDateCopy, $oEndDateCopy);
        $query = "
            INSERT INTO
                {$conf['table']['prefix']}{$conf['table'][$tableName]}
                (
                    start_run,
                    end_run,
                    operation_interval,
                    duration,
                    updated_to
                )
            VALUES
                (
                    '".$start->format('%Y-%m-%d %H:%M:%S')."',
                    '".$end->format('%Y-%m-%d %H:%M:%S')."',
                    {$conf['maintenance']['operationInterval']},
                    ".$duration->toSeconds().",
                    '".$updateTo->format('%Y-%m-%d %H:%M:%S')."'
                )";
        MAX::debug("Logging the script run information into $tableName", PEAR_LOG_DEBUG);
        $result = $this->dbh->query($query);
        if(DB::isError($result)) {
            return false;
        } else {
            return $result;
        }
    }
    
    /**
     * A method to return the time that a Maintenance or Forecasting script
     * was last run, and the operation interval that was in use at the time.
     *
     * @param string $tableName The name of script log table
     * @param array  $additionalFields Any additional fields
     * @return array An array containing the time that represents the last time
     *               maintenance {type} was last run (started). Returns an
     *               array of 2 NULLs if maintenance has never been run.
     */
    function getScriptLastRunInfo($tableName, $additionalFields = array(), $orderBy = 'start_run')
    {
        $conf = $GLOBALS['_MAX']['CONF'];
        $this->queryBuilder->reset();
        $this->queryBuilder->setTable($conf['table']['prefix'] . $conf['table'][$tableName]);
        $this->queryBuilder->setSelect('updated_to');
        foreach ($additionalFields as $field) {
            $this->queryBuilder->addSelect($field);
        }
        $this->queryBuilder->setOrder($orderBy, true);
        $this->queryBuilder->setLimit(0, 1);
        MAX::debug('Obtaining the maintenance run information from table: '.$tableName, PEAR_LOG_DEBUG);
        $rows = $this->queryBuilder->getAll();
        return (count($rows) == 1) ? $rows[0] : false;
    }
    
    /**
     * Obtains a database-level lock for the Maintenance Priority Engine process.
     *
     * @param string $lockName  Lock name
     * @return boolean True if lock was obtained, false otherwise.
     */
    function obtainLock($lockName)
    {
        $query = "SELECT GET_LOCK('$lockName', 1) AS 'lock'";
        $result = $this->dbh->query($query);
        $row = $result->fetchRow();
        if ($row['lock'] == 1) {
            // Lock obtained successfully
            return true;
        }
        return false;
    }

    /**
     * Releases the database-level lock for the Maintenance Priority Engine process.
     *
     * @param string $lockName  Lock name
     * @return mixed True if lock was released, a PEAR Error otherwise.
     */
    function releaseLock($lockName)
    {
        $query = "SELECT RELEASE_LOCK('$lockName')";
        return $this->dbh->query($query);
    }
    
    /**
     * Returns an array of ad limitation hashes for banner or channel limitations.
     *
        Array
        (
            [0] => Array
                (
                    [$primaryKey] => 3
                    [logical] => and
                    [type] => date
                    [comparison] => !=
                    [data] => 2005-5-25
                    [executionorder] => 0
                )
        
            [1] => Array
         ...
     *
     * @param array $fields
     * @param array $wheres
     * @param array $joins
     * @param array $orderBys
     * @param string $primaryKey  Either bannerid or channelid
     * @param string $tableName   Either acls or acls_channel
     * @return array
     */
    function getLimitations($fields = array(), $wheres = array(), $joins = array(), $orderBys = array(), $tableName, $primaryKey)
    {
        $conf = $GLOBALS['_MAX']['CONF'];
        $table = $conf['table']['prefix'] . $tableName;
        
        $this->queryBuilder->reset();
        $this->queryBuilder->setTable($table);
        $this->queryBuilder->setSelect("$table.$primaryKey");
        $this->queryBuilder->addSelect("$table.logical");
        $this->queryBuilder->addSelect("$table.type");
        $this->queryBuilder->addSelect("$table.comparison");
        $this->queryBuilder->addSelect("$table.data");
        $this->queryBuilder->addSelect("$table.executionorder");

        //  where conditions
        foreach ($wheres as $where) {
            list($constraint, $operator) = $where;
            $this->queryBuilder->addWhere($constraint, $operator);
        }
        
        //  joins
        foreach ($joins as $join) {
            list($table, $joinCond) = $join;
            $this->queryBuilder->addJoin($table, $joinCond);
        }
        return  $this->queryBuilder->getAll();
    }
    
}

?>